@@ -31,7 +31,6 @@ from bzrlib.commands import Command
 from bzrlib.option import _global_option, Option
 from bzrlib.merge import merge_inner
 from bzrlib.revision import NULL_REVISION
-from bzrlib.tree import EmptyTree
 import bzrlib.ui
 import bzrlib.ui.text
 from bzrlib.workingtree import WorkingTree
@@ -424,10 +423,6 @@ def import_version(output_dir, version, fast=False,
             wt = WorkingTree.open(output_dir)
         except (NotBranchError, NoWorkingTree):
             wt = None
-        if wt is None:
-            old_basis = EmptyTree()
-        else:
-            old_basis = wt.basis_tree()
         try:
             for result in iter_import_version(output_dir, ancestors, tempdir,
                     pb=progress_bar,
@@ -445,8 +440,6 @@ def import_version(output_dir, version, fast=False,
                 wt = None
             if wt is not None:
                 wt.set_last_revision(wt.branch.last_revision())
-                merge_inner(wt.branch, wt.basis_tree(), old_basis, 
-                            ignore_zero=True, this_tree=wt)
                 wt.revert([])
     
         finally:
@@ -632,7 +625,8 @@ def iter_import_version(output_dir, ancestors, tempdir, pb, fast=False,
         try:
             if missing_ancestor:
                 # if we want it to be in revision-history, do that here.
-                target_tree.add_pending_merge(revision_id(missing_ancestor))
+                target_tree.set_parent_ids([revision_id(missing_ancestor)],
+                                           allow_leftmost_as_ghost=True)
                 missing_ancestor = None
             for merged_rev in direct_merges:
                 target_tree.add_pending_merge(revision_id(merged_rev))
@@ -728,26 +722,21 @@ def bzr_inventory_data(tree):
     bzr_inv.sort()
     return bzr_inv
 
-_global_option('max-count', type = int)
-class cmd_baz_import_branch(Command):
-    """Import an Arch or Baz branch into a bzr branch.  <BZRTOOLS>"""
-    takes_args = ['to_location', 'from_branch?', 'reuse_history*']
-    takes_options = ['verbose', 'max-count']
 
-    def run(self, to_location, from_branch=None, fast=False, max_count=None,
-            verbose=False, dry_run=False, reuse_history_list=[]):
-        to_location = os.path.realpath(str(to_location))
-        if from_branch is not None:
-            try:
-                from_branch = pybaz.Version(from_branch)
-            except pybaz.errors.NamespaceError:
-                print "%s is not a valid Arch branch." % from_branch
-                return 1
-        if reuse_history_list is None:
-            reuse_history_list = []
-        import_version(to_location, from_branch, 
-                       max_count=max_count, 
-                       reuse_history_from=reuse_history_list)
+def baz_import_branch(to_location, from_branch, fast, max_count, verbose, 
+                      dry_run, reuse_history_list):
+    to_location = os.path.realpath(str(to_location))
+    if from_branch is not None:
+        try:
+            from_branch = pybaz.Version(from_branch)
+        except pybaz.errors.NamespaceError:
+            print "%s is not a valid Arch branch." % from_branch
+            return 1
+    if reuse_history_list is None:
+        reuse_history_list = []
+    import_version(to_location, from_branch, 
+                   max_count=max_count, 
+                   reuse_history_from=reuse_history_list)
 
 
 class NotInABranch(Exception):
@@ -756,39 +745,18 @@ class NotInABranch(Exception):
         self.path = path
 
 
-class cmd_baz_import(Command):
-    """Import an Arch or Baz archive into a bzr repository.  <BZRTOOLS>
 
-    This command should be used on local archives (or mirrors) only.  It is
-    quite slow on remote archives.
-    
-    reuse_history allows you to specify any previous imports you 
-    have done of different archives, which this archive has branches
-    tagged from. This will dramatically reduce the time to convert 
-    the archive as it will not have to convert the history already
-    converted in that other branch.
-
-    If you specify prefixes, only branches whose names start with that prefix
-    will be imported.  Skipped branches will be listed, so you can import any
-    branches you missed by accident.  Here's an example of doing a partial
-    import from thelove@canonical.com:
-    bzr baz-import thelove thelove@canonical.com --prefixes dists:talloc-except
-    """
-    takes_args = ['to_root_dir', 'from_archive', 'reuse_history*']
-    takes_options = ['verbose', Option('prefixes', type=str,
-                     help="Prefixes of branches to import, colon-separated")]
-
-    def run(self, to_root_dir, from_archive, verbose=False,
-            reuse_history_list=[], prefixes=None):
-        if reuse_history_list is None:
-            reuse_history_list = []
-        to_root = str(os.path.realpath(to_root_dir))
-        if not os.path.exists(to_root):
-            os.mkdir(to_root)
-        if prefixes is not None:
-            prefixes = prefixes.split(':')
-        import_archive(to_root, from_archive, verbose,
-                       reuse_history_list, prefixes=prefixes)
+def baz_import(to_root_dir, from_archive, verbose=False, reuse_history_list=[],
+               prefixes=None):
+    if reuse_history_list is None:
+        reuse_history_list = []
+    to_root = str(os.path.realpath(to_root_dir))
+    if not os.path.exists(to_root):
+        os.mkdir(to_root)
+    if prefixes is not None:
+        prefixes = prefixes.split(':')
+    import_archive(to_root, from_archive, verbose,
+                   reuse_history_list, prefixes=prefixes)
 
 
 def import_archive(to_root, from_archive, verbose,
