@@ -1,14 +1,27 @@
-#!/usr/bin/python
-
+from bzrlib.diff import _patch_header_date
 import bzrlib.tests
 import os.path
 
+try:
+    from bzrlib.plugins.bzrtools.hunk_selector import (
+        ShelveHunkSelector,
+        UnshelveHunkSelector,
+        )
+    from bzrlib.plugins.bzrtools.errors import NoColor
+except ImportError:
+    from bzrtools.hunk_selector import (
+        ShelveHunkSelector,
+        UnshelveHunkSelector,
+        )
+    from bzrtools.errors import NoColor
+
+
 class ShelfTests(bzrlib.tests.TestCaseWithTransport):
     ORIGINAL = '\n\nhello test world\n\n'
     MODIFIED = '\n\ngoodbye test world\n\n'
     DIFF_HEADER = "=== modified file '%(filename)s'\n"
-    DIFF_1 = """--- %(filename)s\t
-+++ %(filename)s\t
+    DIFF_1 = """--- %(filename)s\t%(old_date)s
++++ %(filename)s\t%(new_date)s
 @@ -1,4 +1,4 @@
  
  
@@ -16,8 +29,8 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
 +goodbye test world
  
 """
-    DIFF_2 = """--- test_file\t
-+++ test_file\t
+    DIFF_2 = """--- test_file\t%(old_date)s
++++ test_file\t%(new_date)s
 @@ -1,4 +1,4 @@
  
  
@@ -26,14 +39,28 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
  
 """
     def _check_diff(self, diff=DIFF_1, filename='test_file'):
-        keys = { 'filename' : filename }
+        old_tree = self.tree.basis_tree()
+        old_date = _patch_header_date(old_tree, 
+                                      old_tree.inventory.path2id(filename),
+                                      filename)
+        new_date = _patch_header_date(self.tree, 
+                                      self.tree.inventory.path2id(filename),
+                                      filename)
+        keys = { 'filename' : filename , 'old_date': old_date, 
+                 'new_date': new_date}
         hdr  = self.DIFF_HEADER % keys
         diff = diff % keys
         self.assertEqual(self.capture('diff', retcode=1), hdr + diff + '\n')
 
-    def _check_shelf(self, idx, diff=DIFF_1, filename='test_file'):
-        diff = diff % { 'filename' : filename }
-        shelf = open(os.path.join(self.tree.branch.base,
+    def _check_shelf(self, idx, diff=DIFF_1, filename='test_file',
+                     new_date=None):
+        old_tree = self.tree.basis_tree()
+        old_date = _patch_header_date(old_tree, 
+                                      old_tree.inventory.path2id(filename),
+                                      filename)
+        diff = diff % { 'filename' : filename, 'old_date': old_date,
+                        'new_date': new_date}
+        shelf = open(os.path.join(self.tree.basedir,
                 '.shelf/shelves/default/' + idx)).read()
         shelf = shelf[shelf.index('\n') + 1:] # skip the message
         self.assertEqual(shelf, diff)
@@ -52,9 +79,15 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
             count -= 1
 
             # Modify the test file
-            file('test_file', 'w').write(self.MODIFIED)
+            # write in binary mode because on win32 line-endings should be LF
+            f = file('test_file', 'wb')
+            f.write(self.MODIFIED)
+            f.close()
 
             self._check_diff()
+            
+            new_date = _patch_header_date(self.tree, 
+                self.tree.inventory.path2id('test_file'), 'test_file')
 
             # Shelve the changes
             self.run_bzr('shelve', '--all', retcode=0)
@@ -65,7 +98,7 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
             # Make sure the file is actually back the way it was
             self.assertEqual(file('test_file').read(), self.ORIGINAL)
 
-            self._check_shelf('00')
+            self._check_shelf('00', new_date=new_date)
 
             # Unshelve
             self.run_bzr('unshelve', '--all', retcode=0)
@@ -73,7 +106,7 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
             self._check_diff()
 
             # Check the shelved patch was backed up
-            self._check_shelf('00~')
+            self._check_shelf('00~', new_date=new_date)
 
             # Make sure the file is back the way it should be
             self.assertEqual(file('test_file').read(), self.MODIFIED)
@@ -91,7 +124,8 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
             self.fail("Shelf exists, but it shouldn't")
 
     def __create_and_add_test_file(self, filename='test_file'):
-        f = open(filename, 'w')
+        # write in binary mode because on win32 line-endings should be LF
+        f = open(filename, 'wb')
         f.write(self.ORIGINAL)
         f.close()
         self.tree.add(self.tree.relpath(os.path.join(os.getcwd(), filename)))
@@ -145,7 +179,7 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
         self.assertEqual(self.capture('diff', retcode=0), '')
 
         # Check the shelf is right
-        shelf = open(os.path.join(self.tree.branch.base,
+        shelf = open(os.path.join(self.tree.basedir,
                     '.shelf/shelves/default', patch)).read()
         self.assertTrue('patch %s' % patch in shelf)
 
@@ -161,7 +195,7 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
         self.__test_show(self.tree, '02')
 
         # Now check we can show a previously shelved patch
-        shelf = open(os.path.join(self.tree.branch.base,
+        shelf = open(os.path.join(self.tree.basedir,
                     '.shelf/shelves/default/00')).read()
         self.assertTrue('patch 00' in shelf)
 
@@ -169,6 +203,22 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
         shown = self.capture('shelf show 00', retcode=0)
         self.assertEqual(shown, shelf)
 
+    def test_shelf_show_unspecified(self):
+        self.tree = self.make_branch_and_tree('.')
+        self.__create_and_add_test_file()
+        self.__test_show(self.tree, '00')
+        self.__test_show(self.tree, '01')
+        self.__test_show(self.tree, '02')
+
+        # Check that not specifying at patch gets us the most recent
+        shelf = open(os.path.join(self.tree.basedir,
+                    '.shelf/shelves/default/02')).read()
+        self.assertTrue('patch 02' in shelf)
+
+        # Check the shown output is right
+        shown = self.capture('shelf show', retcode=0)
+        self.assertEqual(shown, shelf)
+
     def test_shelf_show_with_no_patch(self):
         self.tree = self.make_branch_and_tree('.')
         stderr = self.run_bzr_captured(['shelf', 'show', '00'], retcode=None)[1]
@@ -186,7 +236,7 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
         self.run_bzr('shelve', '--all', retcode=0)
 
         # Write an unapplyable patch into the shelf
-        shelf = open(os.path.join(self.tree.branch.base,
+        shelf = open(os.path.join(self.tree.basedir,
                     '.shelf/shelves/default/00'), 'w')
         shelf.write(self.DIFF_2)
         shelf.close()
@@ -195,7 +245,7 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
         self.run_bzr('unshelve', '--all', retcode=3)
 
         # Make sure the patch is still there, eventhough it's broken
-        shelf = open(os.path.join(self.tree.branch.base,
+        shelf = open(os.path.join(self.tree.basedir,
                     '.shelf/shelves/default/00')).read()
         self.assertEqual(shelf, self.DIFF_2)
 
@@ -254,8 +304,8 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
         self.assertTrue(os.path.exists('.shelf/shelves/default/01~'))
 
         # Check ls works
-        list = self.capture('shelf ls', retcode=0).split('\n')
-        for line in list:
+        lines = self.capture('shelf ls', retcode=0).split('\n')
+        for line in lines:
             self.assertFalse(line.startswith(' 01'))
 
         # Unshelve, if unshelve is confused by the backup it will fail
@@ -268,14 +318,21 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
         self.__create_and_add_test_file(filename='test_file2')
 
         # Modify the test files
-        file('test_file', 'w').write(self.MODIFIED)
-        file('test_file2', 'w').write(self.MODIFIED)
+        # write in binary mode because on win32 line-endings should be LF
+        f = file('test_file', 'wb')
+        f.write(self.MODIFIED)
+        f.close()
+        f = file('test_file2', 'wb')
+        f.write(self.MODIFIED)
+        f.close()
+        new_date = _patch_header_date(self.tree, 
+            self.tree.inventory.path2id('test_file'), 'test_file')
 
         # Shelve the changes
         self.run_bzr('shelve', '--all', 'test_file', retcode=0)
         self.run_bzr('shelve', '--all', 'test_file2', retcode=0)
 
-        self._check_shelf('00')
+        self._check_shelf('00', new_date=new_date)
 
         # Delete 00
         self.run_bzr('shelf', 'delete', '00', retcode=0)
@@ -286,11 +343,11 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
         self.assertTrue(os.path.exists('.shelf/shelves/default/01'))
 
         # Check the backup is right
-        self._check_shelf('00~')
+        self._check_shelf('00~', new_date=new_date)
 
         # Check ls works
-        list = self.capture('shelf ls', retcode=0).split('\n')
-        for line in list:
+        lines = self.capture('shelf ls', retcode=0).split('\n')
+        for line in lines:
             self.assertFalse(line.startswith(' 00'))
 
         # Unshelve should unshelve 01
@@ -363,8 +420,16 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
         # Run a benign shelf command to setup .shelf for us
         self.run_bzr('shelf', 'ls', retcode=0)
 
+        old_tree = self.tree.basis_tree()
+        old_date = _patch_header_date(old_tree, 
+                                      old_tree.inventory.path2id('test_file'),
+                                      'test_file')
+        new_date = _patch_header_date(self.tree, 
+                                      self.tree.inventory.path2id('test_file'),
+                                      'test_file')
         # Fake a -p0 shelved patch
-        diff = self.DIFF_1 % { 'filename' : 'test_file' }
+        diff = self.DIFF_1 % { 'filename' : 'test_file', 'old_date': old_date,
+                               'new_date' : new_date}
         diff = diff.replace('--- ', '--- a/')
         diff = diff.replace('+++ ', '+++ b/')
         open('.shelf/shelves/default/00', 'w').write(diff)
@@ -385,7 +450,10 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
         self.__create_and_add_test_file()
 
         # Modify the test file
-        file('test_file', 'w').write(self.MODIFIED)
+        # write in binary mode because on win32 line-endings should be LF
+        f = file('test_file', 'wb')
+        f.write(self.MODIFIED)
+        f.close()
 
         # Shelve the changes
         self.run_bzr('shelve', '--all', retcode=0)
@@ -405,3 +473,134 @@ class ShelfTests(bzrlib.tests.TestCaseWithTransport):
     def test_shelf_shelf_bogus_subcommand(self):
         self.tree = self.make_branch_and_tree('.')
         self.run_bzr('shelf', 'foo', retcode=3) # <- retcode == 3
+
+    def test_shelf_OOO_unshelve(self):
+        self.tree = self.make_branch_and_tree('.')
+
+        for i in range(1, 5):
+            self.__create_and_add_test_file(filename='test_file%d' % i)
+
+        # Modify the test files
+        for i in range(1, 5):
+            file('test_file%d' % i, 'w').write(self.MODIFIED)
+
+        # Shelve the changes
+        for i in range(1, 5):
+            self.run_bzr('shelve', '--all', 'test_file%d' % i, retcode=0)
+
+        # Check shelving worked
+        for i in range(1, 5):
+            self.assertEqual(file('test_file%d' % i).read(), self.ORIGINAL)
+
+        # We should now have 00-03
+        for i in range(0, 4):
+            self.assertTrue(os.path.exists('.shelf/shelves/default/0%d' % i))
+
+        # Unshelve 00
+        self.run_bzr('unshelve', '--all', '00', retcode=0)
+        self.assertEqual(file('test_file1').read(), self.MODIFIED)
+
+        # Check ls works
+        lines = self.capture('shelf ls', retcode=0).split('\n')
+        for line in lines:
+            self.assertFalse(line.startswith(' 00'))
+
+        # Check we can reshelve once we've unshelved out of order, should be 04
+        self.assertFalse(os.path.exists('.shelf/shelves/default/04'))
+        self.run_bzr('shelve', '--all')
+        self.assertTrue(os.path.exists('.shelf/shelves/default/04'))
+
+        # Check ls works
+        text = self.capture('shelf ls', retcode=0)
+        for line in text.split('\n'):
+            self.assertFalse(line.startswith(' 00'))
+
+        # We now have 01,02,03,04
+        # Unshelve 02
+        self.run_bzr('unshelve', '--all', '02', retcode=0)
+        self.assertEqual(file('test_file3').read(), self.MODIFIED)
+
+        # Unshelve the default, this is the reshelved 00, hence modifies file 1
+        self.run_bzr('unshelve', '--all', retcode=0)
+        self.assertEqual(file('test_file1').read(), self.MODIFIED)
+
+    def test_shelf_switch_basic(self):
+        self.tree = self.make_branch_and_tree('.')
+        self.__create_and_add_test_file()
+
+        # This should go to "default"
+        file('test_file', 'w').write(self.MODIFIED)
+        self.run_bzr('shelve', '--all', 'test_file', retcode=0)
+
+        # Switch to "other"
+        self.run_bzr('shelf', 'switch', 'other', retcode=0)
+        file('test_file', 'w').write(self.MODIFIED)
+        self.run_bzr('shelve', '--all', 'test_file', retcode=0)
+
+        # Check it worked
+        self.assertTrue(os.path.exists('.shelf/shelves/default/00'))
+        self.assertFalse(os.path.exists('.shelf/shelves/default/01'))
+        self.assertTrue(os.path.exists('.shelf/shelves/other/00'))
+
+        # Switch back
+        self.run_bzr('shelf', 'switch', 'default', retcode=0)
+        file('test_file', 'w').write(self.MODIFIED)
+        self.run_bzr('shelve', '--all', 'test_file', retcode=0)
+
+        # Check that worked
+        self.assertTrue(os.path.exists('.shelf/shelves/default/01'))
+        self.assertFalse(os.path.exists('.shelf/shelves/other/01'))
+
+    def test_shelf_bad_patch_arg(self):
+        self.tree = self.make_branch_and_tree('.')
+
+        # Check the bad arg handling
+        stdout, error = self.run_bzr_captured(['unshelve', '01'], retcode=3)
+        self.assertTrue("Patch '01' doesn't exist on shelf" in error)
+
+        stdout, error = self.run_bzr_captured(['unshelve', 'foo'], retcode=3)
+        self.assertTrue("Invalid patch name 'foo'" in error)
+
+        # Hex and is cracky, so it shouldn't work
+        stdout, error = self.run_bzr_captured(['unshelve', '0x00'], retcode=3)
+        self.assertTrue("Invalid patch name '0x00'" in error)
+
+    def test_color_hunk_selector(self):
+        """Make sure NoColor is raised iff terminal.has_ansi_colors is False"""
+        hs = ShelveHunkSelector([])
+        hs = UnshelveHunkSelector([])
+        try:
+            from bzrlib.plugins.bzrtools import terminal
+        except ImportError:
+            from bzrtools import terminal
+        old_has_ansi_colors = terminal.has_ansi_colors
+        terminal.has_ansi_colors = lambda: False
+        try:
+            self.assertRaises(NoColor, ShelveHunkSelector, [], True)
+            self.assertRaises(NoColor, UnshelveHunkSelector, [], True)
+            terminal.has_ansi_colors = lambda: True
+            hs = ShelveHunkSelector([], color=True)
+            hs = UnshelveHunkSelector([], color=True)
+        finally:
+            terminal.has_ansi_colors = old_has_ansi_colors
+
+    def test_no_color(self):
+        """Ensure --no-color switch can be passed"""
+        self.tree = self.make_branch_and_tree('.')
+
+        subdir = 'subdir'
+        os.mkdir(subdir)
+        self.tree.add(subdir)
+        os.chdir(subdir)
+
+        self.__create_and_add_test_file()
+
+        # Modify the test file
+        # write in binary mode because on win32 line-endings should be LF
+        f = file('test_file', 'wb')
+        f.write(self.MODIFIED)
+        f.close()
+        stdout, error = self.run_bzr_captured(['shelve', '--all', 
+                                               '--no-color'])
+        stdout, error = self.run_bzr_captured(['unshelve', '--all', 
+                                               '--no-color'])
