@@ -16,6 +16,7 @@
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 from bzrlib.tests import TestCaseInTempDir, TestCase
+from bzrlib import repository
 from bzrlib.osutils import has_symlinks
 try:
     import pybaz
@@ -23,12 +24,13 @@ except ImportError:
     pybaz = None
 import os
 try:
+    from bzrtools import cmd_baz_import
     from bzrtools.baz_import import (import_version, revision_id,
-                                     cmd_baz_import, make_archive, map_file_id)
+                                      make_archive, map_file_id)
 except ImportError:
+    from bzrlib.plugins.bzrtools import cmd_baz_import 
     from bzrlib.plugins.bzrtools.baz_import import (import_version, 
                                                     revision_id,
-                                                    cmd_baz_import,
                                                     make_archive,
                                                     map_file_id)
 import shutil
@@ -135,8 +137,8 @@ class BazTreeResource(TestResource):
         os.mkdir(os.path.join(self._tmpdir, 'tree'))
         tree = pybaz.init_tree(os.path.join(self._tmpdir, 'tree'),
                                self._import_symlink)
-        os.symlink('missing-file-name',
-                   os.path.join(self._tmpdir, 'tree', 'alink'))
+        link_path = os.path.join(self._tmpdir, 'tree', 'alink')
+        os.symlink('missing-file-name', link_path)
         tree.add_tag('alink')
         id_file = open(os.path.join(tree, '.arch-ids', 'alink.id'), 'w')
         id_file.write('symlink_tag\n')
@@ -144,6 +146,13 @@ class BazTreeResource(TestResource):
         msg = tree.log_message()
         msg["summary"] = "Import with a symlink"
         tree.import_(msg)
+        os.unlink(link_path)
+        f = file(link_path, 'w')
+        f.write('Not a symlink no more!')
+        f.close()
+        msg = tree.log_message()
+        msg["summary"] = "Turn a symlink into a file"
+        tree.commit(msg)
         shutil.rmtree(os.path.join(self._tmpdir, 'tree'))
 
     def make_empty_import(self):
@@ -385,16 +394,18 @@ class TestImportBranch(TestCaseInTempDir):
         # self.assertEqual(branch.missing_revisions(branch3), [])
         
     def test_import_symlink(self):
-        import_version('output', pybaz.Version(self._baz._import_symlink))
+        import_version('output', pybaz.Version(self._baz._import_symlink), 
+                       max_count=1)
         # expected results:
-        # one commit, no files, revision identifier of 
+        # two commits, no files, revision identifier of 
         # 'demo@DONOTUSE_c--import--0--base-0'
         branch = Branch.open('output')
         self.assertEqual(branch.revision_history(),
                          [self._baz._import_symlink_bzr])
         rev = branch.repository.get_revision(self._baz._import_symlink_bzr)
         # and again.
-        import_version('output2', pybaz.Version(self._baz._import_symlink))
+        import_version('output2', pybaz.Version(self._baz._import_symlink),
+                       max_count=1)
         branch2 = Branch.open('output2')
         self.assertEqual(branch.revision_history(), branch2.revision_history())
         rev2 = branch2.repository.get_revision(self._baz._import_symlink_bzr)
@@ -413,6 +424,10 @@ class TestImportBranch(TestCaseInTempDir):
         self.assertEqual(entry.kind, 'symlink')
         self.assertEqual(entry.symlink_target, 'missing-file-name')
 
+        # current bzr doesn't handle type changes
+        self.assertRaises(AssertionError, import_version, 'output3',
+                          pybaz.Version(self._baz._import_symlink))
+
     def test_missing_ancestor(self):
         import_version('output', pybaz.Version(self._baz._missing_ancestor))
         # expected results:
@@ -688,7 +703,7 @@ class TestImport(TestCaseInTempDir):
         shutil.rmtree(os.path.join(self._tmpdir, 'tree'))
 
     def test_cmd_exists(self):
-        from bzrlib.plugins.bzrtools.baz_import import cmd_baz_import
+        from bzrlib.plugins.bzrtools import cmd_baz_import
 
     def test_empty_archive(self):
         command = cmd_baz_import()
@@ -707,8 +722,13 @@ class TestImport(TestCaseInTempDir):
                                        'c','+trunk')))
         self.failUnless(os.path.exists(os.path.join(self._tmpdir,'output', 
                                                     'c1', '0.2','branch')))
+        default_format = repository.RepositoryFormat.get_default_format()
+        if getattr(default_format, 'rich_root_data', False):
+            num_files = 21
+        else:
+            num_files = 20
         walk_len = len(list(os.walk(os.path.join(self._tmpdir,'output'))))
-        self.assertEqual(20, walk_len)
+        self.assertEqual(num_files, walk_len)
 
     def test_run_twice(self):
         self.make_import('c--0')
